LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_SIGNED.ALL;
	
	ENTITY ball IS

	PORT(
	SIGNAL RGB : OUT STD_LOGIC_VECTOR( 15 DOWNTO 0 );
	SIGNAL Vert_sync : IN STD_LOGIC;
	SIGNAL pixel_row, pixel_column : IN STD_LOGIC_VECTOR( 9 DOWNTO 0 ));
	END ball;

	ARCHITECTURE behavior OF ball IS
		-- Video Display Signals
		SIGNAL reset, Ball_on, Direction : STD_LOGIC;
		SIGNAL Size : STD_LOGIC_VECTOR( 9 DOWNTO 0 );
		SIGNAL Ball_Y_motion : STD_LOGIC_VECTOR( 9 DOWNTO 0 );
		SIGNAL Ball_Y_pos, Ball_X_pos : STD_LOGIC_VECTOR( 9 DOWNTO 0 );

	BEGIN 			-- Size of Ball
		Size <= CONV_STD_LOGIC_VECTOR (8,10 );
					-- Ball center X address
		Ball_X_pos <= CONV_STD_LOGIC_VECTOR( 320,10 );
					-- Colors for pixel data on video signal
					-- Turn off Green and Blue to make
					-- color Red when displaying ball
		RGB(15 downto 11) <= "11111"; 	--red



	Display_Ball: PROCESS ( Ball_X_pos, Ball_Y_pos, pixel_column, pixel_row, Size )
		BEGIN 		-- check row & column for ball area
					-- Set Ball_on = '1' to display ball
		IF (Ball_on = '1') THEN
			RGB(10 downto 5)<= "000000";--green
			RGB(4 downto 0) <= "00000";	--blue 
		ELSE
			RGB(10 downto 5)<= "111111";--green
			RGB(4 downto 0) <= "11111";	--blue 
		END IF;
			
		
		IF ( '0' & Ball_X_pos <= pixel_column + Size ) AND
		( Ball_X_pos + Size >= '0' & pixel_column ) AND
		( '0' & Ball_Y_pos <= pixel_row + Size ) AND
		( Ball_Y_pos + Size >= '0' & pixel_row ) THEN
			Ball_on <= '1';
		ELSE
			Ball_on <= '0';
		END IF;
	END PROCESS Display_Ball;


	Move_Ball: PROCESS
		BEGIN
		-- Move ball once every vertical sync
		WAIT UNTIL Vert_sync'EVENT AND Vert_sync = '1';
		-- Bounce off top or bottom of screen
			IF ('0' & Ball_Y_pos) >= CONV_STD_LOGIC_VECTOR(480,10) - Size THEN
				Ball_Y_motion <= CONV_STD_LOGIC_VECTOR(-2,10);
			ELSIF Ball_Y_pos <= Size THEN
				Ball_Y_motion <= CONV_STD_LOGIC_VECTOR(2,10);
			END IF;
			-- Compute next ball Y position
			Ball_Y_pos <= Ball_Y_pos + Ball_Y_motion;
	END PROCESS Move_Ball;

END behavior;